
-- Attempt at cartoon shader
--
-- Copyright (c) 2001-2004 Nick Trout
-- This file is part of Doris. (http://doris.sf.net/)
-- Doris is free software. See terms of GNU license in docs.
-- $Id: cartoon.lua,v 1.2 2004/02/25 02:42:51 trout Exp $

drawLines = 1
drawFront = 1
rotatemat = Matrix:new() -- create a matrix to rotate scene

function cubes(n,t)
    glColor( t.cols[n] )
    glutSolidCube(n)
    if n <= t.depth then
        local d = t.size * (.5 ^ n)
        glPushMatrix()
            glTranslate(d,0,0); cubes(n+1,t)
        glPopMatrix()
    end
end

function init()
  glClearColor (1,1,1,1)
  glDisable (GL_LIGHTING)
  glEnable (GL_CULL_FACE)
  glEnable (GL_LINE_SMOOTH) -- anti alias lines
  glHint (GL_LINE_SMOOTH_HINT, GL_NICEST)
  glEnable (GL_BLEND)
	glBlendFunc (GL_SRC_ALPHA ,GL_ONE_MINUS_SRC_ALPHA)
    -- make object to render
    CallList:create("obj")
    cubes(1, { 
            depth=2, 
            cols={ {.7,0,0,1},{.5,0,0,1} }, 
            size=3 
        } )
    CallList:endList()
end

-- The callback function which renders our scene.
function render()
  glClear(GL_DEPTH_BUFFER_BIT + GL_COLOR_BUFFER_BIT)
  gluLookAt(0,0,10, 0,0,0, 0,1,0)
  
  glMatrixMode(GL_MODELVIEW)
  glPushMatrix()
    glLoadMatrix(rotatemat)

    if drawLines then    
      glPolygonMode(GL_BACK, GL_LINE)
	  glCullFace(GL_FRONT)
	  glDepthFunc(GL_LEQUAL)
	  glLineWidth(4)
      glColor(0,0,0,1)
      CallList:draw("obj")
    end
    
    if drawFront then
      glDepthFunc(GL_LESS)
	  glCullFace(GL_BACK)
	  glPolygonMode(GL_BACK, GL_FILL)
      glLineWidth(1)
      glColor(.7,0,0,1)
      CallList:draw("obj")
    end

  glPopMatrix()
end

-- Create our window and specify a name and window rendering function
wmain = Window:create{
  title = "Doris: Cartoon shader",
  render = render
}

sw=SubWindow{ parent=wmain, side="bottom" }

Rotator{ parent=sw, text="Rotate scene", value=rotatemat, line="|", spin=0.99 }

Checkbox{ parent=sw, text="Draw silhoutte lines", value=drawLines,
    update=function(b)
        if b==1 then drawLines=1 else drawLines=nil end
    end
}

Checkbox{ parent=sw, text="Draw solid front", value=drawFront, line="|",
    update=function(b)
        if b==1 then drawFront=1 else drawFront=nil end
    end
}

init()

