
-- Test all of the features of Doris
--
-- Copyright (c) 2001-2004 Nick Trout
-- This file is part of Doris. (http://doris.sf.net/)
-- Doris is free software. See terms of GNU license in docs.
-- $Id: test.lua,v 1.9 2004/02/25 02:42:51 trout Exp $

-- objects to be displayed and manipulated
sphere = { solid=1,segments=8,rotmat=Matrix:new() }
teapot = { solid=1,rotmat=Matrix:new() }
scene = { rotmat=Matrix:new(), scalemat=Matrix:new() }
lights = { rotmat=Matrix:new() }
scene.scalemat:makeScale(.2,.2,.2)

-- precompile a version of the scene to help performance
function CompileScene()
  CallList:clear()

  CallList:create("sphere")
  Quadric:sphere(sphere)
  CallList:endList()

  CallList:create("teapot")
  Quadric:teapot(teapot)
  CallList:endList()
end

-- The render callback to draw the scene.
function RenderScene()
  glClear(GL_DEPTH_BUFFER_BIT + GL_COLOR_BUFFER_BIT)
  gluLookAt(0,0,3, 0,0,0, 0,1,0);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

  glPushMatrix()
    -- scene
    glLoadMatrix(scene.scalemat)
    glMultMatrix(scene.rotmat)
    glutWireCube(7)

    -- sphere
    glPushMatrix()
      glTranslate(3,0,0)
      glMultMatrix(sphere.rotmat)
      CallList:draw("sphere")
    glPopMatrix()

    -- teapot
    glPushMatrix()
      glMultMatrix(teapot.rotmat)
      CallList:draw("teapot")
    glPopMatrix()

  glPopMatrix()
end

-- Create the main window.
wmain = Window:create{
  title = "Nicks test application",
  render = RenderScene
}

-- AFTER we have created the main window we can compile the scene.
-- We have to wait until the GL objects have been created by the window.
glClearColor(.9,.9,.9,1.0)
CompileScene()

-- sub window at right of screen containing options
sw=SubWindow{ parent=wmain, side="right" }

-- Rollout containing sphere props
ro=Rollout{ parent=sw, text="Sphere:", open=1 }

Checkbox{ parent=ro, text="Solid", value=sphere.solid,
	update = function(x)
	  sphere.solid = x
	  print("solid sphere"..sphere.solid)
	  if sphere.solid==0 then sphere.style="line" else sphere.style="fill" end
	  CompileScene()
	end
}

Spinner{ parent=ro, text="Segments", side="right",
	value=sphere.segments, type="int", limits={3,60},
	update = function(x)
	  sphere.slices,sphere.stacks=x,x
	  print("sphere segments "..x)
	  CompileScene()
	end
}

Spinner{ parent=ro, text="Scale", side="right",
	type="float", limits={.2,4} }

-- Light properties in a rollout
ro=Rollout{ parent=sw, text="Lights", open=0 }

pan=Panel{ parent=ro, text="Light 1" }

Checkbox{ parent=pan,text="Enabled",value=1,
    update=function(x) light0:Switch(x) end
}

Listbox{ parent=pan, text="Colour:",items=mycols,
	update= function(n)
	  c=colvals[n]
	  light0:Diffuse(c[1],c[2],c[3])
	end
}

pan=Panel{ parent=ro, text="Light 2" }
Checkbox{ parent=pan, text="Enabled",value=1,
    update = function(x) light1:Switch(x) end }
Listbox{ parent=pan, text="Colour:",items=mycols,
	update = function(n)
	  c=colvals[n]
	  light1:Diffuse(c[1],c[2],c[3])
	end
}

-- Options rollout
ro=Rollout{ parent=sw, text="Options", open=1, space="h" }
	Checkbox{ parent=ro, text="Draw sphere" }
	Checkbox{ parent=ro, text="Draw teapot" }
	Checkbox{ parent=ro, text="Draw axes" }
	Checkbox{ parent=ro, text="Draw text" }

Listbox{ parent=sw, text="Text:", space="h",
	items={"Hello Nick","La la la","1,2,3,4,5...","Yabba dabba doo" },
	update = function(x) print("List "..x) end
}
Button{ parent=sw, text="Disable movement" }
Button{ parent=sw, text="Enable movement" }
Button{ parent=sw, text="Hide" }
Button{ parent=sw, text="Show" }
Button{ parent=sw, text="Quit",update=function() print("Quit") exit(0) end }

-- 3 rotators at bottom of screen
sw=SubWindow{ parent=wmain, side="bottom" }
  Rotator{ parent=sw, text="Scene:", value=scene.rotmat, line="v" }
  Rotator{ parent=sw, text="Sphere:", value=sphere.rotmat, line="v" }
  Rotator{ parent=sw, text="Lights:", value=lights.rotmat, line="v" }


