
-- Texturing in GL
--
-- Copyright (c) 2001-2004 Nick Trout
-- This file is part of Doris. (http://doris.sf.net/)
-- Doris is free software. See terms of GNU license in docs.
-- $Id: texture.lua,v 1.5 2004/02/25 04:55:41 trout Exp $

rotatemat = Matrix:new() -- create scene rotation matrix

-- Load a texture and use it to texture some objects
--
function createEarth()
  -- Load texture
  print("Loading texture...")
  img = Image:new()
  img:load("res/earth.tga")
  print("Image size: " .. img:getWidth() .. " x " .. img:getHeight())

  -- set texture attributes
  glEnable(GL_TEXTURE_2D)
  glBindTexture(GL_TEXTURE_2D,texs[1])
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB,
      img:getWidth(), img:getHeight(), 0,
      GL_RGB, GL_UNSIGNED_BYTE, img:getPixels())

      -- check to see if we had any errors setting all that up
  _glErrorCheck("after earth texture created")

  reflect = Matrix:new()
  reflect[0] = -1

  -- create a calllist to render our object
  CallList:create("earth")
  glBindTexture(GL_TEXTURE_2D,texs[1])
  glPushMatrix()
    glMultMatrix(reflect)
    glRotate(90,1,0,0)
    --glRotate(180,0,1,0)
    Quadric:sphere{ texture=1,radius=1,slices=16,stacks=16 }
  glPopMatrix()
  CallList:endList()

  CallList:create("tex")
  glBindTexture(GL_TEXTURE_2D,texs[1])
  glBegin(GL_QUADS)
    glTexCoord(0.0, 1.0); glVertex(-1.0, -1.0, 0);
    glTexCoord(1.0, 1.0); glVertex( 1.0, -1.0,  0);
    glTexCoord(1.0, 0.0); glVertex( 1.0,  1.0,  0);
    glTexCoord(0.0, 0.0); glVertex(-1.0,  1.0,  0);
  glEnd()
  CallList:endList()
end

-- Create our own texture and texture some objects
--
function createTeapot()
  print("Creating texture...")
  local size = 64
  tex = gllCreateBuffer(size*size*3,GL_BYTE)
  for i=0,size-1 do
    for j=0,size-1 do
      local c = math.mod(math.floor(i/8) + math.floor(j/8),2)*255
      gllSetBuffer(tex,(i+j*size)*3,c)
    end
  end

  -- Set up textures
  glBindTexture(GL_TEXTURE_2D,texs[2])
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
  glTexParameter(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
  glTexImage2D(GL_TEXTURE_2D, 0, 1,
      size,size, 0,
      GL_RGB, GL_UNSIGNED_BYTE, tex)
  _glErrorCheck("after own texture creation")

  CallList:create("teapot")
  glBindTexture(GL_TEXTURE_2D,texs[2])
  Quadric:teapot{solid=1}
  CallList:endList()

  CallList:create("sphere")
  glBindTexture(GL_TEXTURE_2D,texs[2])
  Quadric:sphere{texture=1, solid=1, slices=10, stacks=10 }
  CallList:endList()
end

function createScene()
  -- create new textures
  texs = {0,0}
  glGenTextures(2,texs)

  createEarth()
  createTeapot()
end

-- The callback function which renders our scene.
--
function renderScene()
  -- Clear the screen and depth buffer
  glClear(GL_DEPTH_BUFFER_BIT + GL_COLOR_BUFFER_BIT)

  -- Tell the camera/eye what to look at
  gluLookAt(0,0,6, 0,0,0, 0,1,0)
  glMatrixMode(GL_MODELVIEW)

  -- Draw the scene.
  glPushMatrix()
    glLoadMatrix(rotatemat)
    --glutWireCube(5)
    glTranslate(2,0,0)
    CallList:draw("earth")
    glTranslate(-4,0,0)
    CallList:draw("tex")
    glTranslate(2,2,0)
    CallList:draw("teapot")
    glTranslate(0,-4,0)
    CallList:draw("sphere")
  glPopMatrix()

  _glErrorCheck("after render")
end

-- Create our window and specify a name and window rendering function
--
wmain = Window:create{
  title = "Doris: Texturing",
  render = renderScene -- specify the redraw function for this window
}

createScene()

-- Put a sub window at the bottom of the screen
--
sw = SubWindow { parent=wmain, side="bottom" }

-- Add an object rotator to the bottom of the screen that
-- manipulates the matrix we created.
--
ro = Rotator {
    parent=sw, -- parent of this widget
    text="Rotate scene", -- text label for this widget
    value=rotatemat, -- matrix that rotator manipulates
    line="|", -- vertical spacing line drawn
    spin=0.99 -- allow the rotator to spin with little damping when released
}


